/*
 * Filename:    NMAZZ.mq4
 * Author:      CaHbI4, based on the original by Tartan at the link below
 * Date:        Mar 14, 2006
 *
 * Description: 
 *
 * Version:     1.0
 *              Initial release
 *              http://www.arkworldmarket.ru/forum/showthread.php?t=966&page=108&pp=10
 */
 
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Yellow
#property indicator_color2 Gold

//---- input parameters
extern string TimeFrame       = "Current time frame";
extern int    depth           = 100;      //ZigZag params
extern int    deviation       = 5;        //ZigZag params
extern int    backstep        = 2;        //ZigZag params
extern int    per             = 50;       //period for CCI and WPR
extern int    ucci            = 200;      //CCI level
extern int    mwpr            = -15;      //WPR low level
extern int    bwpr            = -85;      //WPR high level
extern int    nBars           = 0;        //set 300 for backtestextern bool   
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;



//---- buffers
double ExtMapBuffer1[];
double RevPoint[];
double sig[];

//---- variables
double zz=0, zzold=0,fzz=0;
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(3);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,3,indicator_color1);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexEmptyValue(0,0.0);
   SetIndexLabel(0, "MAZZ");
   
   SetIndexStyle(1,DRAW_ARROW,EMPTY,EMPTY,indicator_color2);
   SetIndexBuffer(1,RevPoint);
   SetIndexArrow(1, 161); //point symbol
   SetIndexLabel(1, "RevPoint");
   
   SetIndexBuffer(2,sig);
   
   ObjectCreate("ZZ_txt",OBJ_TEXT,0,0,0);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
           if (timeFrame!=Period())
           {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);               
                  int x = iBarShift(NULL,timeFrame,Time[i+1]);               
                     ExtMapBuffer1[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",depth,deviation,backstep,per,ucci,mwpr,bwpr,nBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
                     RevPoint[i] = EMPTY_VALUE;
                     if (x!=y)
                        RevPoint[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",depth,deviation,backstep,per,ucci,mwpr,bwpr,nBars,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
               }
               return(0);                  
            }

   //
   //
   //
   //
   //
  
   for(int shift = limit; shift >= 0; shift--)
   {
      if( zz!=0 && zz!=zzold) zzold=zz;
      double ccione = iCCI(NULL,0,per,PRICE_WEIGHTED,shift+2);
      double ccinul = iCCI(NULL,0,per,PRICE_WEIGHTED,shift+1);
      double wprone = iWPR(NULL,0,per,shift+2);
      double wprnul = iWPR(NULL,0,per,shift+1);
             zz     = iCustom(NULL,0,"ZigZagE",depth,deviation,backstep,nBars,0,shift+2);
             
      sig[shift]=0;
      if(zz!=0) fzz=zz; 
      if(zz!=0 && (((ccione<-ucci || ccinul<-ucci) && (wprone<bwpr || wprnul<bwpr)) || 
        ((ccione>ucci || ccinul>ucci) && (wprone>mwpr || wprnul>mwpr)))) sig[shift]=1; else sig[shift]=0;
        
      if(sig[shift]==1) RevPoint[shift]=zz;
      else              RevPoint[shift]=sig[shift];
      
      if( zz==0) fzz=zzold;
      if( zz!=0){
         ObjectSetText("ZZ_txt", "ZZ="+DoubleToStr(zz,4),8,"Arial",White);
         if( zz>Close[0]) ObjectMove("ZZ_txt",0,Time[0],zz+5*Point);
         if( zz<Close[0]) ObjectMove("ZZ_txt",0,Time[0],zz-5*Point);
      }
      ExtMapBuffer1[shift]=fzz;
      }
      
      //
      //
      //
      //
      //
      
      if (alertsOn)
      {
         if (alertsOnCurrent)
              int whichBar = 0;
         else     whichBar = 1;               
         if (sig[whichBar] != sig[whichBar+1])
         if (sig[whichBar] == 1)
               doAlert("trend");
         else  doAlert("notrend");       
      }
      return(0);
   }      

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message = StringConcatenate(" ",Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," NMAZZ ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," NMAZZ "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}
   
   
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   ObjectDelete("ZZ_txt");
}
//+------------------------------------------------------------------+
//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

   